@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2013-2021 NVIDIA Corporation
+    Copyright (c) 2013-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,8 @@
 
 *******************************************************************************/
 
-#ifndef __UVM_COMMON_H__
-#define __UVM_COMMON_H__
+#ifndef _UVM_COMMON_H
+#define _UVM_COMMON_H
 
 #ifdef DEBUG
     #define UVM_IS_DEBUG() 1
@@ -282,9 +282,6 @@ static inline void kmem_cache_destroy_safe(struct kmem_cache **ppCache)
     }
 }
 
-static const uid_t UVM_ROOT_UID = 0;
-
-
 typedef struct
 {
     NvU64 start_time_ns;
@@ -335,7 +332,6 @@ NV_STATUS errno_to_nv_status(int errnoCode);
 int nv_status_to_errno(NV_STATUS status);
 unsigned uvm_get_stale_process_id(void);
 unsigned uvm_get_stale_thread_id(void);
-NvBool uvm_user_id_security_check(uid_t euidTarget);
 
 extern int uvm_enable_builtin_tests;
 
@@ -413,40 +409,4 @@ static inline void uvm_touch_page(struct page *page)
 // Return true if the VMA is one used by UVM managed allocations.
 bool uvm_vma_is_managed(struct vm_area_struct *vma);
 
-static bool uvm_platform_uses_canonical_form_address(void)
-{
-    if (NVCPU_IS_PPC64LE)
-        return false;
-
-    return true;
-}
-
-// Similar to the GPU MMU HAL num_va_bits(), it returns the CPU's num_va_bits().
-static NvU32 uvm_cpu_num_va_bits(void)
-{
-    return fls64(TASK_SIZE - 1) + 1;
-}
-
-// Return the unaddressable range in a num_va_bits-wide VA space, [first, outer)
-static void uvm_get_unaddressable_range(NvU32 num_va_bits, NvU64 *first, NvU64 *outer)
-{
-    UVM_ASSERT(num_va_bits < 64);
-    UVM_ASSERT(first);
-    UVM_ASSERT(outer);
-
-    if (uvm_platform_uses_canonical_form_address()) {
-        *first = 1ULL << (num_va_bits - 1);
-        *outer = (NvU64)((NvS64)(1ULL << 63) >> (64 - num_va_bits));
-    }
-    else {
-        *first = 1ULL << num_va_bits;
-        *outer = ~0Ull;
-    }
-}
-
-static void uvm_cpu_get_unaddressable_range(NvU64 *first, NvU64 *outer)
-{
-    return uvm_get_unaddressable_range(uvm_cpu_num_va_bits(), first, outer);
-}
-
-#endif /* __UVM_COMMON_H__ */
+#endif /* _UVM_COMMON_H */
