@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2022 NVIDIA Corporation
+    Copyright (c) 2015-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -57,16 +57,14 @@
 
 typedef struct
 {
-    // Number of faults from this uTLB that have been fetched but have not been
-    // serviced yet.
+    // Number of faults from this uTLB that have been fetched but have not been serviced yet
     NvU32 num_pending_faults;
 
     // Whether the uTLB contains fatal faults
     bool has_fatal_faults;
 
-    // We have issued a replay of type START_ACK_ALL while containing fatal
-    // faults. This puts the uTLB in lockdown mode and no new translations are
-    // accepted.
+    // We have issued a replay of type START_ACK_ALL while containing fatal faults. This puts
+    // the uTLB in lockdown mode and no new translations are accepted
     bool in_lockdown;
 
     // We have issued a cancel on this uTLB
@@ -128,8 +126,8 @@ struct uvm_service_block_context_struct
         struct list_head service_context_list;
 
         // A mask of GPUs that need to be checked for ECC errors before the CPU
-        // fault handler returns, but after the VA space lock has been unlocked
-        // to avoid the RM/UVM VA space lock deadlocks.
+        // fault handler returns, but after the VA space lock has been unlocked to
+        // avoid the RM/UVM VA space lock deadlocks.
         uvm_processor_mask_t gpus_to_check_for_ecc;
 
         // This is set to throttle page fault thrashing.
@@ -162,14 +160,14 @@ struct uvm_service_block_context_struct
 
     struct
     {
-        // Per-processor mask with the pages that will be resident after
-        // servicing. We need one mask per processor because we may coalesce
-        // faults that trigger migrations to different processors.
+        // Per-processor mask with the pages that will be resident after servicing.
+        // We need one mask per processor because we may coalesce faults that
+        // trigger migrations to different processors.
         uvm_page_mask_t new_residency;
     } per_processor_masks[UVM_ID_MAX_PROCESSORS];
 
     // State used by the VA block routines called by the servicing routine
-    uvm_va_block_context_t block_context;
+    uvm_va_block_context_t *block_context;
 
     // Prefetch state hint
     uvm_perf_prefetch_hint_t prefetch_hint;
@@ -181,28 +179,23 @@ struct uvm_service_block_context_struct
 typedef struct
 {
     // Mask of read faulted pages in a UVM_VA_BLOCK_SIZE aligned region of a SAM
-    // VMA. Used for batching ATS faults in a vma. This is unused for access
-    // counter service requests.
+    // VMA. Used for batching ATS faults in a vma.
     uvm_page_mask_t read_fault_mask;
 
     // Mask of write faulted pages in a UVM_VA_BLOCK_SIZE aligned region of a
-    // SAM VMA. Used for batching ATS faults in a vma. This is unused for access
-    // counter service requests.
+    // SAM VMA. Used for batching ATS faults in a vma.
     uvm_page_mask_t write_fault_mask;
 
     // Mask of successfully serviced pages in a UVM_VA_BLOCK_SIZE aligned region
-    // of a SAM VMA. Used to return ATS fault status. This is unused for access
-    // counter service requests.
+    // of a SAM VMA. Used to return ATS fault status.
     uvm_page_mask_t faults_serviced_mask;
 
     // Mask of successfully serviced read faults on pages in write_fault_mask.
-    // This is unused for access counter service requests.
     uvm_page_mask_t reads_serviced_mask;
 
-    // Mask of all accessed pages in a UVM_VA_BLOCK_SIZE aligned region of a SAM
-    // VMA. This is used as input for access counter service requests and output
-    // of fault service requests.
-    uvm_page_mask_t accessed_mask;
+    // Mask of all faulted pages in a UVM_VA_BLOCK_SIZE aligned region of a
+    // SAM VMA. This is used as input to the prefetcher.
+    uvm_page_mask_t faulted_mask;
 
     // Client type of the service requestor.
     uvm_fault_client_type_t client_type;
@@ -301,8 +294,11 @@ struct uvm_fault_service_batch_context_struct
 
 struct uvm_ats_fault_invalidate_struct
 {
-    bool            tlb_batch_pending;
-    uvm_tlb_batch_t tlb_batch;
+    // Whether the TLB batch contains any information
+    bool            write_faults_in_batch;
+
+    // Batch of TLB entries to be invalidated
+    uvm_tlb_batch_t write_faults_tlb_batch;
 };
 
 typedef struct
@@ -447,9 +443,20 @@ struct uvm_access_counter_service_batch_context_struct
         NvU32                             num_notifications;
 
         // Boolean used to avoid sorting the fault batch by instance_ptr if we
-        // determine at fetch time that all the access counter notifications in
-        // the batch report the same instance_ptr
+        // determine at fetch time that all the access counter notifications in the
+        // batch report the same instance_ptr
         bool is_single_instance_ptr;
+
+        // Scratch space, used to generate artificial physically addressed notifications.
+        // Virtual address notifications are always aligned to 64k. This means up to 16
+        // different physical locations could have been accessed to trigger one notification.
+        // The sub-granularity mask can correspond to any of them.
+        struct
+        {
+            uvm_processor_id_t resident_processors[16];
+            uvm_gpu_phys_address_t phys_addresses[16];
+            uvm_access_counter_buffer_entry_t phys_entry;
+        } scratch;
     } virt;
 
     struct
@@ -460,8 +467,8 @@ struct uvm_access_counter_service_batch_context_struct
         NvU32                              num_notifications;
 
         // Boolean used to avoid sorting the fault batch by aperture if we
-        // determine at fetch time that all the access counter notifications in
-        // the batch report the same aperture
+        // determine at fetch time that all the access counter notifications in the
+        // batch report the same aperture
         bool                              is_single_aperture;
     } phys;
 
@@ -471,9 +478,6 @@ struct uvm_access_counter_service_batch_context_struct
     // Structure used to coalesce access counter servicing in a VA block
     uvm_service_block_context_t block_service_context;
 
-    // Structure used to service access counter migrations in an ATS block.
-    uvm_ats_fault_context_t ats_context;
-
     // Unique id (per-GPU) generated for tools events recording
     NvU32 batch_id;
 };
@@ -660,8 +664,8 @@ struct uvm_gpu_struct
     struct
     {
         // Big page size used by the internal UVM VA space
-        // Notably it may be different than the big page size used by a user's
-        // VA space in general.
+        // Notably it may be different than the big page size used by a user's VA
+        // space in general.
         NvU32 internal_size;
     } big_page;
 
@@ -687,8 +691,8 @@ struct uvm_gpu_struct
         // lazily-populated array of peer GPUs, indexed by the peer's GPU index
         uvm_gpu_t *peer_gpus[UVM_ID_MAX_GPUS];
 
-        // Leaf spinlock used to synchronize access to the peer_gpus table so
-        // that it can be safely accessed from the access counters bottom half
+        // Leaf spinlock used to synchronize access to the peer_gpus table so that
+        // it can be safely accessed from the access counters bottom half
         uvm_spinlock_t peer_gpus_lock;
     } peer_info;
 
@@ -824,8 +828,6 @@ struct uvm_gpu_struct
     {
         NvU32 swizz_id;
 
-        uvmGpuSessionHandle rm_session_handle;
-
         // RM device handle used in many of the UVM/RM APIs.
         //
         // Do not read this field directly, use uvm_gpu_device_handle instead.
@@ -979,10 +981,6 @@ struct uvm_parent_gpu_struct
 
     bool plc_supported;
 
-    // If true, page_tree initialization pre-populates no_ats_ranges. It only
-    // affects ATS systems.
-    bool no_ats_range_required;
-
     // Parameters used by the TLB batching API
     struct
     {
@@ -1054,16 +1052,14 @@ struct uvm_parent_gpu_struct
     // Interrupt handling state and locks
     uvm_isr_info_t isr;
 
-    // Fault buffer info. This is only valid if supports_replayable_faults is
-    // set to true.
+    // Fault buffer info. This is only valid if supports_replayable_faults is set to true
     uvm_fault_buffer_info_t fault_buffer_info;
 
     // PMM lazy free processing queue.
     // TODO: Bug 3881835: revisit whether to use nv_kthread_q_t or workqueue.
     nv_kthread_q_t lazy_free_q;
 
-    // Access counter buffer info. This is only valid if
-    // supports_access_counters is set to true.
+    // Access counter buffer info. This is only valid if supports_access_counters is set to true
     uvm_access_counter_buffer_info_t access_counter_buffer_info;
 
     // Number of uTLBs per GPC. This information is only valid on Pascal+ GPUs.
@@ -1113,7 +1109,7 @@ struct uvm_parent_gpu_struct
     uvm_rb_tree_t instance_ptr_table;
     uvm_spinlock_t instance_ptr_table_lock;
 
-    // This is set to true if the GPU belongs to an SLI group.
+    // This is set to true if the GPU belongs to an SLI group. Else, set to false.
     bool sli_enabled;
 
     struct
@@ -1140,8 +1136,8 @@ struct uvm_parent_gpu_struct
     // environment, rather than using the peer-id field of the PTE (which can
     // only address 8 gpus), all gpus are assigned a 47-bit physical address
     // space by the fabric manager. Any physical address access to these
-    // physical address spaces are routed through the switch to the
-    // corresponding peer.
+    // physical address spaces are routed through the switch to the corresponding
+    // peer.
     struct
     {
         bool is_nvswitch_connected;
@@ -1351,7 +1347,7 @@ static NvU64 uvm_gpu_retained_count(uvm_gpu_t *gpu)
 void uvm_parent_gpu_kref_put(uvm_parent_gpu_t *gpu);
 
 // Calculates peer table index using GPU ids.
-NvU32 uvm_gpu_peer_table_index(uvm_gpu_id_t gpu_id1, uvm_gpu_id_t gpu_id2);
+NvU32 uvm_gpu_peer_table_index(const uvm_gpu_id_t gpu_id0, const uvm_gpu_id_t gpu_id1);
 
 // Either retains an existing PCIe peer entry or creates a new one. In both
 // cases the two GPUs are also each retained.
@@ -1366,12 +1362,11 @@ void uvm_gpu_release_pcie_peer_access(uvm_gpu_t *gpu0, uvm_gpu_t *gpu1);
 // They must not be the same gpu.
 uvm_aperture_t uvm_gpu_peer_aperture(uvm_gpu_t *local_gpu, uvm_gpu_t *remote_gpu);
 
-// Get the processor id accessible by the given GPU for the given physical
-// address.
+// Get the processor id accessible by the given GPU for the given physical address
 uvm_processor_id_t uvm_gpu_get_processor_id_by_address(uvm_gpu_t *gpu, uvm_gpu_phys_address_t addr);
 
 // Get the P2P capabilities between the gpus with the given indexes
-uvm_gpu_peer_t *uvm_gpu_index_peer_caps(uvm_gpu_id_t gpu_id1, uvm_gpu_id_t gpu_id2);
+uvm_gpu_peer_t *uvm_gpu_index_peer_caps(const uvm_gpu_id_t gpu_id0, const uvm_gpu_id_t gpu_id1);
 
 // Get the P2P capabilities between the given gpus
 static uvm_gpu_peer_t *uvm_gpu_peer_caps(const uvm_gpu_t *gpu0, const uvm_gpu_t *gpu1)
@@ -1379,10 +1374,10 @@ static uvm_gpu_peer_t *uvm_gpu_peer_caps(const uvm_gpu_t *gpu0, const uvm_gpu_t
     return uvm_gpu_index_peer_caps(gpu0->id, gpu1->id);
 }
 
-static bool uvm_gpus_are_nvswitch_connected(uvm_gpu_t *gpu1, uvm_gpu_t *gpu2)
+static bool uvm_gpus_are_nvswitch_connected(const uvm_gpu_t *gpu0, const uvm_gpu_t *gpu1)
 {
-    if (gpu1->parent->nvswitch_info.is_nvswitch_connected && gpu2->parent->nvswitch_info.is_nvswitch_connected) {
-        UVM_ASSERT(uvm_gpu_peer_caps(gpu1, gpu2)->link_type >= UVM_GPU_LINK_NVLINK_2);
+    if (gpu0->parent->nvswitch_info.is_nvswitch_connected && gpu1->parent->nvswitch_info.is_nvswitch_connected) {
+        UVM_ASSERT(uvm_gpu_peer_caps(gpu0, gpu1)->link_type >= UVM_GPU_LINK_NVLINK_2);
         return true;
     }
 
@@ -1464,9 +1459,9 @@ NV_STATUS uvm_gpu_check_ecc_error(uvm_gpu_t *gpu);
 
 // Check for ECC errors without calling into RM
 //
-// Calling into RM is problematic in many places, this check is always safe to
-// do. Returns NV_WARN_MORE_PROCESSING_REQUIRED if there might be an ECC error
-// and it's required to call uvm_gpu_check_ecc_error() to be sure.
+// Calling into RM is problematic in many places, this check is always safe to do.
+// Returns NV_WARN_MORE_PROCESSING_REQUIRED if there might be an ECC error and
+// it's required to call uvm_gpu_check_ecc_error() to be sure.
 NV_STATUS uvm_gpu_check_ecc_error_no_rm(uvm_gpu_t *gpu);
 
 // Map size bytes of contiguous sysmem on the GPU for physical access
@@ -1523,13 +1518,11 @@ bool uvm_gpu_can_address(uvm_gpu_t *gpu, NvU64 addr, NvU64 size);
 // The GPU must be initialized before calling this function.
 bool uvm_gpu_can_address_kernel(uvm_gpu_t *gpu, NvU64 addr, NvU64 size);
 
-bool uvm_platform_uses_canonical_form_address(void);
-
 // Returns addr's canonical form for host systems that use canonical form
 // addresses.
 NvU64 uvm_parent_gpu_canonical_address(uvm_parent_gpu_t *parent_gpu, NvU64 addr);
 
-static bool uvm_gpu_is_coherent(const uvm_parent_gpu_t *parent_gpu)
+static bool uvm_parent_gpu_is_coherent(const uvm_parent_gpu_t *parent_gpu)
 {
     return parent_gpu->system_bus.memory_window_end > parent_gpu->system_bus.memory_window_start;
 }
@@ -1571,9 +1564,8 @@ uvm_aperture_t uvm_gpu_page_tree_init_location(const uvm_gpu_t *gpu);
 // Debug print of GPU properties
 void uvm_gpu_print(uvm_gpu_t *gpu);
 
-// Add the given instance pointer -> user_channel mapping to this GPU. The
-// bottom half GPU page fault handler uses this to look up the VA space for GPU
-// faults.
+// Add the given instance pointer -> user_channel mapping to this GPU. The bottom
+// half GPU page fault handler uses this to look up the VA space for GPU faults.
 NV_STATUS uvm_gpu_add_user_channel(uvm_gpu_t *gpu, uvm_user_channel_t *user_channel);
 void uvm_gpu_remove_user_channel(uvm_gpu_t *gpu, uvm_user_channel_t *user_channel);
 
