@@ -280,9 +280,7 @@ NV_STATUS uvm_va_space_mm_register(uvm_va_space_t *va_space)
         }
     }
 
-    if ((UVM_IS_CONFIG_HMM() || UVM_HMM_RANGE_FAULT_SUPPORTED()) &&
-        uvm_va_space_pageable_mem_access_supported(va_space)) {
-
+    if ((UVM_IS_CONFIG_HMM() || UVM_ATS_PREFETCH_SUPPORTED()) && uvm_va_space_pageable_mem_access_supported(va_space)) {
         #if UVM_CAN_USE_MMU_NOTIFIERS()
             // Initialize MMU interval notifiers for this process. This allows
             // mmu_interval_notifier_insert() to be called without holding the
@@ -318,17 +316,6 @@ void uvm_va_space_mm_unregister(uvm_va_space_t *va_space)
     if (!va_space_mm->mm)
         return;
 
-    // At this point the mm is still valid because uvm_mm_release()
-    // hasn't yet called mmput(). uvm_hmm_va_space_destroy() will kill
-    // all the va_blocks along with any associated gpu_chunks, so we
-    // need to make sure these chunks are free. However freeing them
-    // requires a valid mm so we can call migrate_vma_setup(), so we
-    // do that here.
-    // TODO: Bug 3902536: [UVM-HMM] add code to migrate GPU memory
-    // without having a va_block
-    if (uvm_hmm_is_enabled(va_space))
-        uvm_hmm_evict_va_blocks(va_space);
-
     if (uvm_va_space_mm_enabled(va_space)) {
         if (UVM_ATS_IBM_SUPPORTED_IN_DRIVER() && g_uvm_global.ats.enabled)
             uvm_mmu_notifier_unregister(va_space_mm);
