@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1999-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1999-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -782,6 +782,8 @@ inline void NV_API_CALL out_string(const char *str)
     printk("%s", str);
 }
 
+#define NV_PRINT_LOCAL_BUFF_LEN_MAX      530
+
 /*
  * nv_printf() prints to the kernel log for the driver.
  * Returns the number of characters written.
@@ -790,11 +792,38 @@ int NV_API_CALL nv_printf(NvU32 debuglevel, const char *printf_format, ...)
 {
     va_list arglist;
     int chars_written = 0;
+    NvBool bForced = (NV_DBG_FORCE_LEVEL(debuglevel) == debuglevel);
+    debuglevel = debuglevel & 0xff;
 
-    if (debuglevel >= ((cur_debuglevel >> 4) & 0x3))
+    // This function is protected by the "_nv_dbg_lock" lock, so it is still
+    // thread-safe to store the print buffer in a static variable, thus
+    // avoiding a problem with kernel stack size.
+    static char buff[NV_PRINT_LOCAL_BUFF_LEN_MAX];
+
+    /*
+     * Print a message if:
+     * 1. Caller indicates that filtering should be skipped, or
+     * 2. debuglevel is at least cur_debuglevel for DBG_MODULE_OS (bits 4:5). Support for print
+     * modules has been removed with DBG_PRINTF, so this check should be cleaned up.
+     */
+    if (bForced ||
+        (debuglevel >= ((cur_debuglevel >> 4) & 0x3)))
     {
-        size_t length;
-        char *temp;
+        size_t loglevel_length = 0, format_length = strlen(printf_format);
+        size_t length = 0;
+        const char *loglevel = "";
+
+        switch (debuglevel)
+        {
+            case NV_DBG_INFO:       loglevel = KERN_DEBUG; break;
+            case NV_DBG_SETUP:      loglevel = KERN_NOTICE; break;
+            case NV_DBG_WARNINGS:   loglevel = KERN_WARNING; break;
+            case NV_DBG_ERRORS:     loglevel = KERN_ERR; break;
+            case NV_DBG_HW_ERRORS:  loglevel = KERN_CRIT; break;
+            case NV_DBG_FATAL:      loglevel = KERN_CRIT; break;
+        }
+
+        loglevel_length = strlen(loglevel);
 
         // When printk is called to extend the output of the previous line
         // (i.e. when the previous line did not end in \n), the printk call
@@ -814,24 +843,19 @@ int NV_API_CALL nv_printf(NvU32 debuglevel, const char *printf_format, ...)
         // string always contains only one \n (at the end) and NV_PRINTF_EX
         // is deleted.  But that is unlikely to ever happen.
 
-        length = strlen(printf_format);
+        length = loglevel_length + format_length + sizeof(KERN_CONT);
         if (length < 1)
             return 0;
 
-        temp = kmalloc(length + sizeof(KERN_CONT), GFP_ATOMIC);
-        if (!temp)
-            return 0;
-
         // KERN_CONT changed in the 3.6 kernel, so we can't assume its
         // composition or size.
-        memcpy(temp, KERN_CONT, sizeof(KERN_CONT) - 1);
-        memcpy(temp + sizeof(KERN_CONT) - 1, printf_format, length + 1);
+        memcpy(buff, KERN_CONT, sizeof(KERN_CONT) - 1);
+        memcpy(buff + sizeof(KERN_CONT) - 1, loglevel, loglevel_length);
+        memcpy(buff + sizeof(KERN_CONT) - 1 + loglevel_length, printf_format, length + 1);
 
         va_start(arglist, printf_format);
-        chars_written = vprintk(temp, arglist);
+        chars_written = vprintk(buff, arglist);
         va_end(arglist);
-
-        kfree(temp);
     }
 
     return chars_written;
@@ -1197,6 +1221,87 @@ NvBool NV_API_CALL os_is_efi_enabled(void)
     return efi_enabled(EFI_BOOT);
 }
 
+void NV_API_CALL os_get_screen_info(
+    NvU64 *pPhysicalAddress,
+    NvU32 *pFbWidth,
+    NvU32 *pFbHeight,
+    NvU32 *pFbDepth,
+    NvU32 *pFbPitch,
+    NvU64 consoleBar1Address,
+    NvU64 consoleBar2Address
+)
+{
+    *pPhysicalAddress = 0;
+    *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
+
+#if defined(CONFIG_FB) && defined(NV_NUM_REGISTERED_FB_PRESENT)
+    if (num_registered_fb > 0)
+    {
+        int i;
+
+        for (i = 0; i < num_registered_fb; i++)
+        {
+            if (!registered_fb[i])
+                continue;
+
+            /* Make sure base address is mapped to GPU BAR */
+            if ((registered_fb[i]->fix.smem_start == consoleBar1Address) ||
+                (registered_fb[i]->fix.smem_start == consoleBar2Address))
+            {
+                *pPhysicalAddress = registered_fb[i]->fix.smem_start;
+                *pFbWidth = registered_fb[i]->var.xres;
+                *pFbHeight = registered_fb[i]->var.yres;
+                *pFbDepth = registered_fb[i]->var.bits_per_pixel;
+                *pFbPitch = registered_fb[i]->fix.line_length;
+                return;
+            }
+        }
+    }
+#endif
+
+    /*
+     * If the screen info is not found in the registered FBs then fallback
+     * to the screen_info structure.
+     *
+     * The SYSFB_SIMPLEFB option, if enabled, marks VGA/VBE/EFI framebuffers as
+     * generic framebuffers so the new generic system-framebuffer drivers can
+     * be used instead. DRM_SIMPLEDRM drives the generic system-framebuffers
+     * device created by SYSFB_SIMPLEFB.
+     *
+     * SYSFB_SIMPLEFB registers a dummy framebuffer which does not contain the
+     * information required by os_get_screen_info(), therefore you need to
+     * fall back onto the screen_info structure.
+     */
+
+#if NV_IS_EXPORT_SYMBOL_PRESENT_screen_info
+    /*
+     * If there is not a framebuffer console, return 0 size.
+     *
+     * orig_video_isVGA is set to 1 during early Linux kernel
+     * initialization, and then will be set to a value, such as
+     * VIDEO_TYPE_VLFB or VIDEO_TYPE_EFI if an fbdev console is used.
+     */
+    if (screen_info.orig_video_isVGA > 1)
+    {
+        NvU64 physAddr = screen_info.lfb_base;
+#if defined(VIDEO_CAPABILITY_64BIT_BASE)
+        physAddr |= (NvU64)screen_info.ext_lfb_base << 32;
+#endif
+
+        /* Make sure base address is mapped to GPU BAR */
+        if ((physAddr == consoleBar1Address) ||
+            (physAddr == consoleBar2Address))
+        {
+            *pPhysicalAddress = physAddr;
+            *pFbWidth = screen_info.lfb_width;
+            *pFbHeight = screen_info.lfb_height;
+            *pFbDepth = screen_info.lfb_depth;
+            *pFbPitch = screen_info.lfb_linelength;
+        }
+    }
+#endif
+}
+
 void NV_API_CALL os_dump_stack(void)
 {
     dump_stack();
@@ -1726,6 +1831,7 @@ NV_STATUS NV_API_CALL os_open_temporary_file
     void **ppFile
 )
 {
+#if NV_FILESYSTEM_ACCESS_AVAILABLE
 #if defined(O_TMPFILE)
     struct file *file;
     const char *default_path = "/tmp";
@@ -1771,6 +1877,9 @@ NV_STATUS NV_API_CALL os_open_temporary_file
 #else
     return NV_ERR_NOT_SUPPORTED;
 #endif
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
 }
 
 void NV_API_CALL os_close_file
@@ -1778,7 +1887,9 @@ void NV_API_CALL os_close_file
     void *pFile
 )
 {
+#if NV_FILESYSTEM_ACCESS_AVAILABLE
     filp_close(pFile, NULL);
+#endif
 }
 
 #define NV_MAX_NUM_FILE_IO_RETRIES 10
@@ -1791,6 +1902,7 @@ NV_STATUS NV_API_CALL os_write_file
     NvU64 offset
 )
 {
+#if NV_FILESYSTEM_ACCESS_AVAILABLE
     loff_t f_pos = offset;
     ssize_t num_written;
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
@@ -1821,6 +1933,9 @@ retry:
     }
 
     return NV_OK;
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
 }
 
 NV_STATUS NV_API_CALL os_read_file
@@ -1831,6 +1946,7 @@ NV_STATUS NV_API_CALL os_read_file
     NvU64 offset
 )
 {
+#if NV_FILESYSTEM_ACCESS_AVAILABLE
     loff_t f_pos = offset;
     ssize_t num_read;
     int num_retries = NV_MAX_NUM_FILE_IO_RETRIES;
@@ -1861,6 +1977,9 @@ retry:
     }
 
     return NV_OK;
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
 }
 
 NV_STATUS NV_API_CALL os_open_readonly_file
@@ -1869,6 +1988,7 @@ NV_STATUS NV_API_CALL os_open_readonly_file
     void       **ppFile
 )
 {
+#if NV_FILESYSTEM_ACCESS_AVAILABLE
     struct file *file;
 
     /*
@@ -1890,6 +2010,9 @@ NV_STATUS NV_API_CALL os_open_readonly_file
     *ppFile = (void *)file;
 
     return NV_OK;
+#else
+    return NV_ERR_NOT_SUPPORTED;
+#endif
 }
 
 NV_STATUS NV_API_CALL os_open_and_read_file
@@ -2130,8 +2253,6 @@ static int os_numa_verify_gpu_memory_zone(struct notifier_block *nb,
     return NOTIFY_OK;
 }
 
-#define ADD_REMOVE_GPU_MEMORY_NUM_SEGMENTS 4
-
 NV_STATUS NV_API_CALL os_numa_add_gpu_memory
 (
     void *handle,
@@ -2145,12 +2266,7 @@ NV_STATUS NV_API_CALL os_numa_add_gpu_memory
     nv_linux_state_t *nvl = pci_get_drvdata(handle);
     nv_state_t *nv = NV_STATE_PTR(nvl);
     NvU64 base = offset + nvl->coherent_link_info.gpu_mem_pa;
-    int ret = 0;
-    NvU64 memblock_size;
-    NvU64 size_remaining;
-    NvU64 calculated_segment_size;
-    NvU64 segment_size;
-    NvU64 segment_base;
+    int ret;
     os_numa_gpu_mem_hotplug_notifier_t notifier =
     {
         .start_pa = base,
@@ -2183,49 +2299,11 @@ NV_STATUS NV_API_CALL os_numa_add_gpu_memory
         goto failed;
     }
 
-    //
-    // Adding all memory at once can take a long time. Split up memory into segments
-    // with schedule() in between to prevent soft lockups. Memory segments for
-    // add_memory_driver_managed() need to be aligned to memblock size.
-    //
-    // If there are any issues splitting into segments, then add all memory at once.
-    //
-    if (os_numa_memblock_size(&memblock_size) == NV_OK)
-    {
-        calculated_segment_size = NV_ALIGN_UP(size / ADD_REMOVE_GPU_MEMORY_NUM_SEGMENTS, memblock_size);
-    }
-    else
-    {
-        // Don't split into segments, add all memory at once
-        calculated_segment_size = size;
-    }
-
-    segment_size = calculated_segment_size;
-    segment_base = base;
-    size_remaining = size;
-
-    while ((size_remaining > 0) &&
-           (ret == 0))
-    {
-        if (segment_size > size_remaining)
-        {
-            segment_size = size_remaining;
-        }
-
 #ifdef NV_ADD_MEMORY_DRIVER_MANAGED_HAS_MHP_FLAGS_ARG
-        ret = add_memory_driver_managed(node, segment_base, segment_size, "System RAM (NVIDIA)", MHP_NONE);
+    ret = add_memory_driver_managed(node, base, size, "System RAM (NVIDIA)", MHP_NONE);
 #else
-        ret = add_memory_driver_managed(node, segment_base, segment_size, "System RAM (NVIDIA)");
+    ret = add_memory_driver_managed(node, base, size, "System RAM (NVIDIA)");
 #endif
-        nv_printf(NV_DBG_SETUP, "NVRM: add_memory_driver_managed() returns: %d for segment_base: 0x%llx, segment_size: 0x%llx\n",
-                  ret, segment_base, segment_size);
-
-        segment_base += segment_size;
-        size_remaining -= segment_size;
-
-        // Yield CPU to prevent soft lockups
-        schedule();
-    }
     unregister_memory_notifier(&notifier.memory_notifier);
 
     if (ret == 0)
@@ -2239,33 +2317,14 @@ NV_STATUS NV_API_CALL os_numa_add_gpu_memory
             zone_end_pfn(zone) != end_pfn)
         {
             nv_printf(NV_DBG_ERRORS, "NVRM: GPU memory zone movable auto onlining failed!\n");
-
 #ifdef NV_OFFLINE_AND_REMOVE_MEMORY_PRESENT
-            // Since zone movable auto onlining failed, need to remove the added memory.
-            segment_size = calculated_segment_size;
-            segment_base = base;
-            size_remaining = size;
-
-            while (size_remaining > 0)
-            {
-                if (segment_size > size_remaining)
-                {
-                    segment_size = size_remaining;
-                }
-
 #ifdef NV_REMOVE_MEMORY_HAS_NID_ARG
-                ret = offline_and_remove_memory(node, segment_base, segment_size);
+            if (offline_and_remove_memory(node, base, size) != 0)
 #else
-                ret = offline_and_remove_memory(segment_base, segment_size);
+            if (offline_and_remove_memory(base, size) != 0)
 #endif
-                nv_printf(NV_DBG_SETUP, "NVRM: offline_and_remove_memory() returns: %d for segment_base: 0x%llx, segment_size: 0x%llx\n",
-                          ret, segment_base, segment_size);
-
-                segment_base += segment_size;
-                size_remaining -= segment_size;
-
-                // Yield CPU to prevent soft lockups
-                schedule();
+            {
+                nv_printf(NV_DBG_ERRORS, "NVRM: offline_and_remove_memory failed\n");
             }
 #endif
             goto failed;
@@ -2285,77 +2344,6 @@ failed:
     return NV_ERR_NOT_SUPPORTED;
 }
 
-
-typedef struct {
-    NvU64 base;
-    NvU64 size;
-    NvU32 nodeId;
-    int ret;
-} remove_numa_memory_info_t;
-
-static void offline_numa_memory_callback
-(
-    void *args
-)
-{
-#ifdef NV_OFFLINE_AND_REMOVE_MEMORY_PRESENT
-    remove_numa_memory_info_t *pNumaInfo = (remove_numa_memory_info_t *)args;
-    int ret = 0;
-    NvU64 memblock_size;
-    NvU64 size_remaining;
-    NvU64 calculated_segment_size;
-    NvU64 segment_size;
-    NvU64 segment_base;
-
-    //
-    // Removing all memory at once can take a long time. Split up memory into segments
-    // with schedule() in between to prevent soft lockups. Memory segments for
-    // offline_and_remove_memory() need to be aligned to memblock size.
-    //
-    // If there are any issues splitting into segments, then remove all memory at once.
-    //
-    if (os_numa_memblock_size(&memblock_size) == NV_OK)
-    {
-        calculated_segment_size = NV_ALIGN_UP(pNumaInfo->size / ADD_REMOVE_GPU_MEMORY_NUM_SEGMENTS, memblock_size);
-    }
-    else
-    {
-        // Don't split into segments, remove all memory at once
-        calculated_segment_size = pNumaInfo->size;
-    }
-
-    segment_size = calculated_segment_size;
-    segment_base = pNumaInfo->base;
-    size_remaining = pNumaInfo->size;
-
-    while (size_remaining > 0)
-    {
-        if (segment_size > size_remaining)
-        {
-            segment_size = size_remaining;
-        }
-
-#ifdef NV_REMOVE_MEMORY_HAS_NID_ARG
-        ret = offline_and_remove_memory(pNumaInfo->nodeId,
-                                        segment_base,
-                                        segment_size);
-#else
-        ret = offline_and_remove_memory(segment_base,
-                                        segment_size);
-#endif
-        nv_printf(NV_DBG_SETUP, "NVRM: offline_and_remove_memory() returns: %d for segment_base: 0x%llx, segment_size: 0x%llx\n",
-                  ret, segment_base, segment_size);
-        pNumaInfo->ret |= ret;
-
-        segment_base += segment_size;
-        size_remaining -= segment_size;
-
-        // Yield CPU to prevent soft lockups
-        schedule();
-    }
-#endif
-}
-
 NV_STATUS NV_API_CALL os_numa_remove_gpu_memory
 (
     void *handle,
