@@ -529,10 +529,20 @@ typedef struct
 {
     NvBool bLinkErrorsCallBackEnabled;
     NvBool bLinkStateCallBackEnabled;
-    NvBool bResetAndDrainRetry;
+    NvU64  lastRetrainTime;
+    NvU64  lastLinkUpTime;
+} NVLINK_LINK_ERROR_REPORTING_STATE;
 
+typedef struct
+{
     NVLINK_LINK_ERROR_INFO_ERR_MASKS fatalIntrMask;
     NVLINK_LINK_ERROR_INFO_ERR_MASKS nonFatalIntrMask;
+} NVLINK_LINK_ERROR_REPORTING_DATA;
+
+typedef struct
+{
+    NVLINK_LINK_ERROR_REPORTING_STATE state;
+    NVLINK_LINK_ERROR_REPORTING_DATA  data;
 } NVLINK_LINK_ERROR_REPORTING;
 
 typedef struct
@@ -834,7 +844,6 @@ typedef const struct
 #define nvswitch_setup_link_loopback_mode_ls10       nvswitch_setup_link_loopback_mode_lr10
 
 #define nvswitch_link_lane_reversed_ls10             nvswitch_link_lane_reversed_lr10
-#define nvswitch_request_tl_link_state_ls10          nvswitch_request_tl_link_state_lr10
 
 #define nvswitch_i2c_get_port_info_ls10             nvswitch_i2c_get_port_info_lr10
 #define nvswitch_i2c_set_hw_speed_mode_ls10         nvswitch_i2c_set_hw_speed_mode_lr10
@@ -986,6 +995,7 @@ NvlStatus nvswitch_reset_and_drain_links_ls10(nvswitch_device *device, NvU64 lin
 void      nvswitch_service_minion_all_links_ls10(nvswitch_device *device);
 NvlStatus nvswitch_ctrl_get_board_part_number_ls10(nvswitch_device *device, NVSWITCH_GET_BOARD_PART_NUMBER_VECTOR *p);
 void      nvswitch_create_deferred_link_state_check_task_ls10(nvswitch_device *device, NvU32 nvlipt_instance, NvU32 link);
+NvlStatus nvswitch_request_tl_link_state_ls10(nvlink_link *link, NvU32 tlLinkState, NvBool bSync);
 
 //
 // SU generated functions
