@@ -841,8 +841,8 @@ RmInitDeviceDma(
     if (nv->iovaspace_id != NV_IOVA_DOMAIN_NONE)
     {
         OBJSYS *pSys = SYS_GET_INSTANCE();
-        POBJVMM pVmm = SYS_GET_VMM(pSys);
-        POBJVASPACE pIOVAS;
+        OBJVMM *pVmm = SYS_GET_VMM(pSys);
+        OBJVASPACE *pIOVAS;
         NV_STATUS status = vmmCreateVaspace(pVmm, IO_VASPACE_A,
                                             nv->iovaspace_id, 0, 0ULL, ~0ULL,
                                             0ULL, 0ULL,
@@ -865,8 +865,8 @@ RmTeardownDeviceDma(
     if (nv->iovaspace_id != NV_IOVA_DOMAIN_NONE)
     {
         OBJSYS *pSys = SYS_GET_INSTANCE();
-        POBJVMM pVmm = SYS_GET_VMM(pSys);
-        POBJVASPACE pIOVAS;
+        OBJVMM *pVmm = SYS_GET_VMM(pSys);
+        OBJVASPACE *pIOVAS;
 
         if (NV_OK == vmmGetVaspaceFromId(pVmm, nv->iovaspace_id, IO_VASPACE_A, &pIOVAS))
         {
@@ -900,12 +900,14 @@ RmInitNvDevice(
         return;
     }
 
+    os_disable_console_access();
 
     status->rmStatus = gpumgrStateInitGpu(pGpu);
     if (status->rmStatus != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "*** Cannot initialize the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_STATE_INIT_FAILED);
+        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE;
@@ -935,10 +937,13 @@ RmInitNvDevice(
         NV_PRINTF(LEVEL_ERROR,
                   "*** Cannot load state into the device\n");
         RM_SET_ERROR(*status, RM_INIT_GPU_LOAD_FAILED);
+        os_enable_console_access();
         return;
     }
     nvp->flags |= NV_INIT_FLAG_GPU_STATE_LOAD;
 
+    os_enable_console_access();
+
     status->rmStatus = gpuPerformUniversalValidation_HAL(pGpu);
     if (status->rmStatus != NV_OK)
     {
@@ -1155,7 +1160,6 @@ NvBool RmInitPrivateState(
     nv_priv_t *nvp;
     NvU32 gpuId;
     NvU32 pmc_boot_0 = 0;
-    NvU32 pmc_boot_1 = 0;
     NvU32 pmc_boot_42 = 0;
 
     NV_SET_NV_PRIV(pNv, NULL);
@@ -1173,7 +1177,6 @@ NvBool RmInitPrivateState(
         }
 
         pmc_boot_0 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_0);
-        pmc_boot_1 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_1);
         pmc_boot_42 = NV_PRIV_REG_RD32(pNv->regs->map_u, NV_PMC_BOOT_42);
 
         os_unmap_kernel_space(pNv->regs->map_u, os_page_size);
@@ -1213,7 +1216,6 @@ NvBool RmInitPrivateState(
     os_mem_set(nvp, 0, sizeof(*nvp));
     nvp->status = NV_ERR_INVALID_STATE;
     nvp->pmc_boot_0 = pmc_boot_0;
-    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
     NV_SET_NV_PRIV(pNv, nvp);
 
@@ -1232,7 +1234,7 @@ void RmClearPrivateState(
     nv_i2c_adapter_entry_t i2c_adapters[MAX_I2C_ADAPTERS];
     nv_dynamic_power_t dynamicPowerCopy;
     NvU32 x = 0;
-    NvU32 pmc_boot_0, pmc_boot_1, pmc_boot_42;
+    NvU32 pmc_boot_0, pmc_boot_42;
 
     //
     // Do not clear private state after GPU resets, it is used while
@@ -1250,7 +1252,6 @@ void RmClearPrivateState(
     pRegistryCopy = nvp->pRegistry;
     dynamicPowerCopy = nvp->dynamic_power;
     pmc_boot_0 = nvp->pmc_boot_0;
-    pmc_boot_1 = nvp->pmc_boot_1;
     pmc_boot_42 = nvp->pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1266,7 +1267,6 @@ void RmClearPrivateState(
     nvp->pRegistry = pRegistryCopy;
     nvp->dynamic_power = dynamicPowerCopy;
     nvp->pmc_boot_0 = pmc_boot_0;
-    nvp->pmc_boot_1 = pmc_boot_1;
     nvp->pmc_boot_42 = pmc_boot_42;
 
     for (x = 0; x < MAX_I2C_ADAPTERS; x++)
@@ -1537,7 +1537,6 @@ NvBool RmInitAdapter(
     KernelDisplay  *pKernelDisplay;
     const void     *gspFwHandle = NULL;
     const void     *gspFwLogHandle = NULL;
-    NvBool          consoleDisabled = NV_FALSE;
 
     GSP_FIRMWARE    gspFw = {0};
     PORT_UNREFERENCED_VARIABLE(gspFw);
@@ -1565,22 +1564,6 @@ NvBool RmInitAdapter(
 
     pSys = SYS_GET_INSTANCE();
 
-    //
-    // WAR: If the below UEFI property is set, display RM will attempt to read
-    // the state cache during RM init in order to retrieve a snapshot of the
-    // display state that the UEFI driver has already programmed. On Orin
-    // (T234D), the UEFI boot flow is being enabled on Linux, but our UEFI
-    // driver doesn't have any display support right now. As such, our UEFI
-    // driver won't allocate any of the display channels, which means that RM
-    // will attempt to read the state cache for uninitialized channels. WAR this
-    // issue by un-setting the below UEFI property for now.
-    //
-    // JIRA task TDS-5094 tracks adding display support to the UEFI driver.
-    //
-    if (NV_IS_SOC_DISPLAY_DEVICE(nv)) {
-        pSys->setProperty(pSys, PDB_PROP_SYS_IS_UEFI, NV_FALSE);
-    }
-
     //
     // Get firmware from the OS, if requested, and decide if RM will run as a
     // firmware client.
@@ -1646,16 +1629,6 @@ NvBool RmInitAdapter(
 
     RmInitAcpiMethods(pOS, pSys, pGpu);
 
-    //
-    // For GPU driving console, disable console access here, to ensure no console
-    // writes through BAR1 can interfere with physical RM's setup of BAR1
-    //
-    if (rm_get_uefi_console_status(nv))
-    {
-        os_disable_console_access();
-        consoleDisabled = NV_TRUE;
-    }
-
     //
     // If GSP fw RM support is enabled then start the GSP microcode
     // (including the task running the full instance of the RM) and
@@ -1731,12 +1704,6 @@ NvBool RmInitAdapter(
         goto shutdown;
     }
 
-    if (consoleDisabled)
-    {
-        os_enable_console_access();
-        consoleDisabled = NV_FALSE;
-    }
-
     // LOCK: acquire GPUs lock
     status.rmStatus = rmGpuLocksAcquire(GPUS_LOCK_FLAGS_NONE,
                                         RM_LOCK_MODULES_INIT);
@@ -1892,38 +1859,6 @@ NvBool RmInitAdapter(
         }
     }
 
-    {
-        // OpenRM support for features beyond what is used on Data Center GPUs
-        // is still fairly immature, so for now require users to opt into use of
-        // OpenRM with a special registry key, if not on a Data Center GPU.
-        const GspStaticConfigInfo *pSCI = GPU_GET_GSP_STATIC_INFO(pGpu);
-
-        if (pSCI->computeBranding != COMPUTE_BRANDING_TYPE_TESLA &&
-            ((pGpu->idInfo.PCIDeviceID >> 16) & 0xffff) != NV_PCI_DEVID_DEVICE_PG189_SKU600)
-        {
-            NvU32 data = NV_REG_OPENRM_ENABLE_UNSUPPORTED_GPUS_DEFAULT;
-            RmReadRegistryDword(nv, NV_REG_OPENRM_ENABLE_UNSUPPORTED_GPUS, &data);
-
-            if (data == NV_REG_OPENRM_ENABLE_UNSUPPORTED_GPUS_DISABLE)
-            {
-                if (!nv->printed_openrm_enable_unsupported_gpus_error)
-                {
-                    nv_printf(NV_DBG_ERRORS,
-                        "NVRM: Open nvidia.ko is only ready for use on Data Center GPUs.\n");
-                    nv_printf(NV_DBG_ERRORS,
-                        "NVRM: To force use of Open nvidia.ko on other GPUs, see the\n");
-                    nv_printf(NV_DBG_ERRORS,
-                        "NVRM: 'OpenRmEnableUnsupportedGpus' kernel module parameter described\n");
-                    nv_printf(NV_DBG_ERRORS,
-                        "NVRM: in the README.\n");
-                    nv->printed_openrm_enable_unsupported_gpus_error = NV_TRUE;
-                }
-                RM_SET_ERROR(status, RM_INIT_FIRMWARE_INIT_FAILED);
-                goto shutdown;
-            }
-        }
-    }
-
     NV_DEV_PRINTF(NV_DBG_SETUP, nv, "RmInitAdapter succeeded!\n");
 
     retVal = NV_TRUE;
@@ -1932,11 +1867,6 @@ NvBool RmInitAdapter(
  shutdown:
     nv->flags &= ~NV_FLAG_IN_RECOVERY;
 
-    if (consoleDisabled)
-    {
-        os_enable_console_access();
-    }
-
     // call ShutdownAdapter to undo anything we've done above
     RmShutdownAdapter(nv);
 
