@@ -421,8 +421,7 @@ confComputeStateInitLocked_IMPL
     ConfidentialCompute *pConfCompute
 )
 {
-    RM_API *pRmApi = IS_GSP_CLIENT(pGpu) ? GPU_GET_PHYSICAL_RMAPI(pGpu) :
-                                           rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
 
     NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi,
                                            pGpu->hInternalClient,
