@@ -159,8 +159,10 @@ kgspAllocBootArgs_GH100
                                         flags),
                           _kgspAllocBootArgs_exit_cleanup);
 
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_56,
+                    pKernelGsp->pGspFmcArgumentsDescriptor);
     NV_ASSERT_OK_OR_GOTO(nvStatus,
-                          memdescAlloc(pKernelGsp->pGspFmcArgumentsDescriptor),
+                          nvStatus,
                           _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -524,6 +526,7 @@ kgspBootstrapRiscvOSEarly_GH100
 {
     KernelFalcon *pKernelFalcon = staticCast(pKernelGsp, KernelFalcon);
     KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+    KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
     NV_STATUS     status        = NV_OK;
 
     // Only for GSP client builds
@@ -534,7 +537,7 @@ kgspBootstrapRiscvOSEarly_GH100
     }
 
     // Clear ECC errors before attempting to load GSP
-    status = gpuClearEccCounts_HAL(pGpu);
+    status = kmemsysClearEccCounts_HAL(pGpu, pKernelMemorySystem);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "Issue clearing ECC counts! Status:0x%x\n", status);
@@ -653,7 +656,7 @@ exit:
     // If GSP fails to boot, check if there's any DED error.
     if (status != NV_OK)
     {
-        gpuCheckEccCounts_HAL(pGpu);
+        kmemsysCheckEccCounts_HAL(pGpu, pKernelMemorySystem);
     }
     NV_ASSERT(status == NV_OK);
 
