@@ -37,6 +37,7 @@
 #include "os/os.h"
 #include "nverror.h"
 #include "gsp/gsp_error.h"
+#include "nvrm_registry.h"
 #include "crashcat/crashcat_report.h"
 
 #include "published/turing/tu102/dev_gsp.h"
@@ -126,8 +127,9 @@ kgspAllocBootArgs_TU102
                                        flags),
                         _kgspAllocBootArgs_exit_cleanup);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-                         memdescAlloc(pKernelGsp->pWprMetaDescriptor),
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_WPR_METADATA,
+                    pKernelGsp->pWprMetaDescriptor);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus,
                          _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -140,8 +142,6 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pWprMeta = (GspFwWprMeta *)NvP64_VALUE(pVa);
     pKernelGsp->pWprMetaMappingPriv = pPriv;
 
-    portMemSet(pKernelGsp->pWprMeta, 0, sizeof(*pKernelGsp->pWprMeta));
-
     //
     // Setup libos arguments memory
     //
@@ -154,8 +154,9 @@ kgspAllocBootArgs_TU102
                                        flags),
                          _kgspAllocBootArgs_exit_cleanup);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-                         memdescAlloc(pKernelGsp->pLibosInitArgumentsDescriptor),
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_LIBOS_ARGS,
+                    pKernelGsp->pLibosInitArgumentsDescriptor);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus,
                          _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -168,8 +169,6 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pLibosInitArgumentsCached = (LibosMemoryRegionInitArgument *)NvP64_VALUE(pVa);
     pKernelGsp->pLibosInitArgumentsMappingPriv = pPriv;
 
-    portMemSet(pKernelGsp->pLibosInitArgumentsCached, 0, LIBOS_INIT_ARGUMENTS_SIZE);
-
     // Setup bootloader arguments memory.
     NV_ASSERT(sizeof(GSP_ARGUMENTS_CACHED) <= 0x1000);
 
@@ -180,8 +179,9 @@ kgspAllocBootArgs_TU102
                                        flags),
                          _kgspAllocBootArgs_exit_cleanup);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-                         memdescAlloc(pKernelGsp->pGspArgumentsDescriptor),
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_BOOTLOADER_ARGS,
+                    pKernelGsp->pGspArgumentsDescriptor);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus,
                          _kgspAllocBootArgs_exit_cleanup);
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -194,8 +194,6 @@ kgspAllocBootArgs_TU102
     pKernelGsp->pGspArgumentsCached = (GSP_ARGUMENTS_CACHED *)NvP64_VALUE(pVa);
     pKernelGsp->pGspArgumentsMappingPriv = pPriv;
 
-    portMemSet(pKernelGsp->pGspArgumentsCached, 0, sizeof(*pKernelGsp->pGspArgumentsCached));
-
     return nvStatus;
 
 _kgspAllocBootArgs_exit_cleanup:
@@ -498,9 +496,8 @@ kgspGetGspRmBootUcodeStorage_TU102
  *   | GSP FW (non-WPR) HEAP    |
  *   ---------------------------- <- nonWprHeapOffset, gspFwRsvdStart
  *
- *  gspFwHeapOffset** contains the entire WPR heap region, which can be subdivided
- *  for various GSP FW components.
- *
+ *  gspFwHeapOffset** contains the RM/Libos Heap. First 16 Mb are for Libos heap
+ *  rest is for GSP-RM
  * @param       pGpu          GPU object pointer
  * @param       pKernelGsp    KernelGsp object pointer
  * @param       pGspFw        Pointer to GSP-RM fw image.
@@ -521,6 +518,7 @@ kgspCalculateFbLayout_TU102
     NvU64                vbiosReservedOffset;
     NvU64                mmuLockLo, mmuLockHi;
     NvBool               bIsMmuLockValid;
+    NvU32                data;
 
     ct_assert(sizeof(*pWprMeta) == 256);
 
@@ -530,6 +528,8 @@ kgspCalculateFbLayout_TU102
     NV_ASSERT_OR_RETURN(pKernelGsp->gspRmBootUcodeSize != 0, NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pRiscvDesc != NULL, NV_ERR_INVALID_STATE);
 
+    portMemSet(pWprMeta, 0, sizeof *pWprMeta);
+
     NV_ASSERT_OK_OR_RETURN(kmemsysGetUsableFbSize_HAL(pGpu, pKernelMemorySystem, &pWprMeta->fbSize));
 
     //
@@ -563,11 +563,8 @@ kgspCalculateFbLayout_TU102
     else
         vbiosReservedOffset = pWprMeta->vgaWorkspaceOffset;
 
-    // Set the size of the GSP FW ahead of kgspGetWprEndMargin()
-    pWprMeta->sizeOfRadix3Elf = pGspFw->imageSize;
-
-    // End of WPR region (128KB aligned), shifted for any WPR end margin
-    pWprMeta->gspFwWprEnd = NV_ALIGN_DOWN64(vbiosReservedOffset - kgspGetWprEndMargin(pGpu, pKernelGsp), 0x20000);
+    // End of WPR region (128KB aligned)
+    pWprMeta->gspFwWprEnd = NV_ALIGN_DOWN64(vbiosReservedOffset, 0x20000);
 
     pWprMeta->frtsSize = kgspGetFrtsSize(pGpu, pKernelGsp);
     pWprMeta->frtsOffset = pWprMeta->gspFwWprEnd - pWprMeta->frtsSize;
@@ -576,6 +573,9 @@ kgspCalculateFbLayout_TU102
     pWprMeta->sizeOfBootloader = pKernelGsp->gspRmBootUcodeSize;
     pWprMeta->bootBinOffset = NV_ALIGN_DOWN64(pWprMeta->frtsOffset - pWprMeta->sizeOfBootloader, 0x1000);
 
+    // Compute GSP firmware image size
+    pWprMeta->sizeOfRadix3Elf = pGspFw->imageSize;
+
     //
     // Compute the start of the ELF.  Align to 64K to avoid issues with
     // inherent alignment constraints.
@@ -633,11 +633,22 @@ kgspCalculateFbLayout_TU102
         pWprMeta->sizeOfCrashReportQueue = (NvU32)memdescGetSize(pCrashCatQueueMemDesc);
     }
 
+    if ((osReadRegistryDword(pGpu, NV_REG_STR_RM_BOOT_GSPRM_WITH_BOOST_CLOCKS, &data) == NV_OK) &&
+        (data == NV_REG_STR_RM_BOOT_GSPRM_WITH_BOOST_CLOCKS_DISABLED))
+    {
+        pKernelGsp->bBootGspRmWithBoostClocks = NV_FALSE;
+    }
+
     pWprMeta->bootCount = 0;
     pWprMeta->verified = 0;
     pWprMeta->revision = GSP_FW_WPR_META_REVISION;
     pWprMeta->magic = GSP_FW_WPR_META_MAGIC;
 
+    if (pKernelGsp->bBootGspRmWithBoostClocks)
+    {
+        pWprMeta->flags |= GSP_FW_FLAGS_CLOCK_BOOST;
+    }
+
 #if 0
     NV_PRINTF(LEVEL_ERROR, "WPR meta data offset:     0x%016llx\n", pWprMeta->gspFwWprStart);
     NV_PRINTF(LEVEL_ERROR, "  magic:                  0x%016llx\n", pWprMeta->magic);
@@ -773,22 +784,6 @@ kgspResetHw_TU102
     return NV_OK;
 }
 
-static NvBool kgspCrashCatReportImpactsGspRm(CrashCatReport *pReport)
-{
-    NV_CRASHCAT_CONTAINMENT containment;
-
-    containment = crashcatReportSourceContainment_HAL(pReport);
-    switch (containment)
-    {
-       case NV_CRASHCAT_CONTAINMENT_RISCV_MODE_M:
-       case NV_CRASHCAT_CONTAINMENT_RISCV_HART:
-       case NV_CRASHCAT_CONTAINMENT_UNCONTAINED:
-           return NV_TRUE;
-       default:
-           return NV_FALSE;
-    }
-}
-
 NvBool
 kgspHealthCheck_TU102
 (
@@ -807,12 +802,15 @@ kgspHealthCheck_TU102
 
         while ((pReport = crashcatEngineGetNextCrashReport(pCrashCatEng)) != NULL)
         {
-            if (kgspCrashCatReportImpactsGspRm(pReport))
-                bHealthy = NV_FALSE;
+            bHealthy = NV_FALSE;
+
+            pKernelGsp->bFatalError = NV_TRUE;
 
             NV_PRINTF(LEVEL_ERROR,
                 "****************************** GSP-CrashCat Report *******************************\n");
             crashcatReportLog(pReport);
+            NV_PRINTF(LEVEL_ERROR,
+                "**********************************************************************************\n");
 
             objDelete(pReport);
         }
@@ -830,17 +828,24 @@ kgspHealthCheck_TU102
     {
         NvU32 mb1 = GPU_REG_RD32(pGpu, NV_PGSP_MAILBOX(1));
         NvU32 skipped = DRF_VAL(_GSP, _ERROR, _SKIPPED, mb0);
-
         bHealthy = NV_FALSE;
 
+        pKernelGsp->bFatalError = NV_TRUE;
+
         // Clear the mailbox
         GPU_REG_WR32(pGpu, NV_PGSP_MAILBOX(0), 0);
 
         NV_PRINTF(LEVEL_ERROR,
                   "********************************* GSP Failure **********************************\n");
 
-        NV_ERROR_LOG(pGpu, GSP_ERROR,
-                     "GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x.  The GPU likely needs to be reset.",
+        nvErrorLog_va((void*)pGpu, GSP_ERROR,
+                      "GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x.  The GPU likely needs to be reset.",
+                      DRF_VAL(_GSP, _ERROR, _TASK, mb0),
+                      DRF_VAL(_GSP, _ERROR, _CODE, mb0),
+                      DRF_VAL(_GSP, _ERROR, _REASON, mb0),
+                      mb1);
+        NVLOG_PRINTF(NV_PRINTF_MODULE, NVLOG_ROUTE_RM, LEVEL_ERROR, NV_PRINTF_ADD_PREFIX
+                     ("GSP Error: Task %d raised error code 0x%x for reason 0x%x at 0x%x"),
                      DRF_VAL(_GSP, _ERROR, _TASK, mb0),
                      DRF_VAL(_GSP, _ERROR, _CODE, mb0),
                      DRF_VAL(_GSP, _ERROR, _REASON, mb0),
@@ -851,29 +856,16 @@ kgspHealthCheck_TU102
         {
             NV_PRINTF(LEVEL_ERROR, "%d more errors skipped\n", skipped);
         }
+
+        NV_PRINTF(LEVEL_ERROR,
+                  "********************************************************************************\n");
     }
 
 exit_health_check:
     if (!bHealthy)
     {
-        NvBool bFirstFatal = !pKernelGsp->bFatalError;
-
-        pKernelGsp->bFatalError = NV_TRUE;
-
-        if (pKernelGsp->pRpc)
-        {
-            kgspLogRpcDebugInfo(pGpu, pKernelGsp->pRpc, GSP_ERROR, pKernelGsp->bPollingForRpcResponse);
-        }
-
-        if (bFirstFatal)
-        {
-            kgspRcAndNotifyAllUserChannels(pGpu, pKernelGsp, GSP_ERROR);
-        }
-
-        gpuCheckEccCounts_HAL(pGpu);
-
-        NV_PRINTF(LEVEL_ERROR,
-                  "**********************************************************************************\n");
+        KernelMemorySystem *pKernelMemorySystem = GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu);
+        kmemsysCheckEccCounts_HAL(pGpu, pKernelMemorySystem);
     }
     return bHealthy;
 }
@@ -987,6 +979,9 @@ kgspIsWpr2Up_TU102
     return (wpr2HiVal != 0);
 }
 
+#define FWSECLIC_PROG_START_TIMEOUT     50000    // 50ms
+#define FWSECLIC_PROG_COMPLETE_TIMEOUT  2000000  // 2s
+
 NV_STATUS
 kgspWaitForGfwBootOk_TU102
 (
@@ -994,16 +989,51 @@ kgspWaitForGfwBootOk_TU102
     KernelGsp *pKernelGsp
 )
 {
+    NvU32 timeoutUs = FWSECLIC_PROG_START_TIMEOUT + FWSECLIC_PROG_COMPLETE_TIMEOUT;
+    RMTIMEOUT timeout;
     NV_STATUS status = NV_OK;
 
-    status = gpuWaitForGfwBootComplete_HAL(pGpu);
-    if (status != NV_OK)
+    // Use the OS timer since the GPU timer is not ready yet
+    gpuSetTimeout(pGpu, gpuScaleTimeout(pGpu, timeoutUs), &timeout,
+                  GPU_TIMEOUT_FLAGS_OSTIMER);
+
+    while (status == NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "failed to wait for GFW boot complete: 0x%x VBIOS version %s\n",
-                  status, pKernelGsp->vbiosVersionStr);
-        NV_PRINTF(LEVEL_ERROR, "(the GPU may be in a bad state and may need to be reset)\n");
+        //
+        // Before reading the actual GFW_BOOT status register,
+        // we want to check that FWSEC has lowered its PLM first.
+        // If not then obviously it has not completed.
+        //
+        if (GPU_FLD_TEST_DRF_DEF(pGpu,
+                _PGC6,
+                _AON_SECURE_SCRATCH_GROUP_05_PRIV_LEVEL_MASK,
+                _READ_PROTECTION_LEVEL0,
+                _ENABLE)
+            )
+        {
+            if (GPU_FLD_TEST_DRF_DEF(pGpu,
+                    _PGC6,
+                    _AON_SECURE_SCRATCH_GROUP_05_0_GFW_BOOT,
+                    _PROGRESS,
+                    _COMPLETED)
+                )
+            {
+                return NV_OK;
+            }
+        }
+
+        status = gpuCheckTimeout(pGpu, &timeout);
     }
 
+    // The wait failed if we reach here (as above loop returns upon success).
+    NV_PRINTF(LEVEL_ERROR, "failed to wait for GFW_BOOT: 0x%x (progress 0x%x, VBIOS version %s)\n",
+              status, GPU_REG_RD_DRF(pGpu,
+                        _PGC6,
+                        _AON_SECURE_SCRATCH_GROUP_05_0_GFW_BOOT,
+                        _PROGRESS),
+              pKernelGsp->vbiosVersionStr);
+    NV_PRINTF(LEVEL_ERROR, "(the GPU may be in a bad state and may need to be reset)\n");
+
     return status;
 }
 
@@ -1048,7 +1078,7 @@ kgspSavePowerMgmtState_TU102
     gspfwSRMeta.magic                   = GSP_FW_SR_META_MAGIC;
     gspfwSRMeta.revision                = GSP_FW_SR_META_REVISION;
     gspfwSRMeta.sizeOfSuspendResumeData = pKernelGsp->pWprMeta->gspFwWprEnd - pKernelGsp->pWprMeta->gspFwWprStart;
-
+    gspfwSRMeta.flags                   = pKernelGsp->pWprMeta->flags;
 
     NV_ASSERT_OK_OR_GOTO(nvStatus,
                          kgspCreateRadix3(pGpu,
@@ -1073,8 +1103,9 @@ kgspSavePowerMgmtState_TU102
                                        MEMDESC_FLAGS_NONE),
                          exit_fail_cleanup);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-                         memdescAlloc(pKernelGsp->pSRMetaDescriptor),
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_SR_METADATA,
+                    pKernelGsp->pSRMetaDescriptor);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus,
                          exit_fail_cleanup);
 
     // Copy SR Metadata Structure
@@ -1128,36 +1159,3 @@ exit_cleanup:
     kgspFreeSuspendResumeData_HAL(pGpu, pKernelGsp);
     return nvStatus;
 }
-
-void
-kgspReadEmem_TU102
-(
-    KernelGsp *pKernelGsp,
-    NvU64      offset,
-    NvU64      size,
-    void      *pBuf
-)
-{
-    NvU32 ememMask = DRF_SHIFTMASK(NV_PGSP_EMEMC_OFFS) | DRF_SHIFTMASK(NV_PGSP_EMEMC_BLK);
-    OBJGPU *pGpu = ENG_GET_GPU(pKernelGsp);
-    NvU32 limit = size - NVBIT(DRF_SHIFT(NV_PGSP_EMEMC_OFFS));
-    NvU32 *pBuffer = pBuf;
-
-    portMemSet(pBuf, 0, size);
-
-#if defined(DEBUG) || defined(DEVELOP)
-    NV_ASSERT_OR_RETURN_VOID((offset & ~ememMask) == 0);
-    NV_ASSERT_OR_RETURN_VOID(limit <= ememMask);
-    NV_ASSERT_OR_RETURN_VOID(offset + limit <= ememMask);
-#else
-    NV_CHECK_OR_RETURN_VOID(LEVEL_SILENT, (offset & ~ememMask) == 0);
-    NV_CHECK_OR_RETURN_VOID(LEVEL_SILENT, limit <= ememMask);
-    NV_CHECK_OR_RETURN_VOID(LEVEL_SILENT, offset + limit <= ememMask);
-#endif
-
-    GPU_REG_WR32(pGpu, NV_PGSP_EMEMC(pKernelGsp->ememPort),
-                 offset | DRF_DEF(_PGSP, _EMEMC, _AINCR, _TRUE));
-
-    for (NvU32 idx = 0; idx < size / sizeof(NvU32); idx++)
-        pBuffer[idx] = GPU_REG_RD32(pGpu, NV_PGSP_EMEMD(pKernelGsp->ememPort));
-}
