@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2019-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2019-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -256,9 +256,9 @@ GspMsgQueuesInit
 
     memdescSetFlag(pMQCollection->pSharedMemDesc, MEMDESC_FLAGS_KERNEL_MODE, NV_TRUE);
 
-    NV_ASSERT_OK_OR_GOTO(nvStatus,
-        memdescAlloc(pMQCollection->pSharedMemDesc),
-        error_ret);
+    memdescTagAlloc(nvStatus, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_58, 
+                    pMQCollection->pSharedMemDesc);
+    NV_ASSERT_OK_OR_GOTO(nvStatus, nvStatus, error_ret);
 
     // Create kernel mapping for command queue.
     NV_ASSERT_OK_OR_GOTO(nvStatus,
@@ -589,9 +589,7 @@ NV_STATUS GspMsgQueueSendCommand(MESSAGE_QUEUE_INFO *pMQI, OBJGPU *pGpu)
         if (pNextElement == NULL)
         {
             pMQI->txBufferFull++;
-            NV_PRINTF_COND(pMQI->txBufferFull == 1, LEVEL_ERROR, LEVEL_INFO,
-                           "buffer is full (waiting for %d free elements, got %d)\n",
-                           pCQE->elemCount, i);
+            NV_PRINTF_COND(pMQI->txBufferFull == 1, LEVEL_ERROR, LEVEL_INFO, "buffer is full\n");
             nvStatus = NV_ERR_BUSY_RETRY;
             goto done;
         }
