@@ -287,8 +287,8 @@ sec2utilsDestruct_IMPL
 {
     OBJCHANNEL *pChannel = pSec2Utils->pChannel;
     OBJGPU *pGpu = pSec2Utils->pGpu;
-    MemoryManager *pMemoryManager = NULL;
-    RM_API *pRmApi = NULL;
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    RM_API *pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
 
     // Sanity checks
     if ((pGpu == NULL) || (pChannel == NULL))
@@ -302,9 +302,6 @@ sec2utilsDestruct_IMPL
         return;
     }
 
-    pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    pRmApi = rmapiGetInterface(RMAPI_GPU_LOCK_INTERNAL);
-
     ccslContextClear(pSec2Utils->pCcslCtx);
 
     if ((pChannel->bClientUserd) && (pChannel->pControlGPFifo != NULL))
@@ -594,7 +591,7 @@ sec2utilsMemset_IMPL
     do
     {
         NvU64 maxContigSize = bContiguous ? memsetLength : (pageGranularity - offset % pageGranularity);
-        NvU32 memsetSizeContig = (NvU32)NV_MIN(NV_MIN(memsetLength, maxContigSize), NVCBA2_DECRYPT_SCRUB_SIZE_MAX_BYTES);
+        NvU32 memsetSizeContig = (NvU32)NV_MIN(NV_MIN(memsetLength, maxContigSize), NVCBA2_DECRYPT_COPY_SIZE_MAX_BYTES);
 
         channelPbInfo.dstAddr = memdescGetPhysAddr(pMemDesc, AT_GPU, offset);
 
