@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2012-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2012-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -43,7 +43,7 @@
 /* ------------------ static and helper functions prototypes------------------*/
 static NvU32 translateAllocFlagsToVASpaceFlags(NvU32 allocFlags, NvU32 *translatedFlags);
 static NvU32 translatePageSizeToVASpaceFlags(NV_VASPACE_ALLOCATION_PARAMETERS *pNvVASpaceAllocParams);
-static NV_STATUS _vaspaceapiManagePageLevelsForSplitVaSpace(OBJGPU *pGpu, NvHandle hClient, NvU32 gpuMask, NvU32 flags, VASPACEAPI_MANAGE_PAGE_LEVELS_ACTION action);
+static NV_STATUS _vaspaceapiManagePageLevelsForSplitVaSpace(OBJGPU *pGpu, Device *pDevice, NvU32 gpuMask, NvU32 flags, VASPACEAPI_MANAGE_PAGE_LEVELS_ACTION action);
 
 NvBool
 vaspaceapiCanCopy_IMPL(VaSpaceApi *pResource)
@@ -59,14 +59,12 @@ vaspaceapiConstruct_IMPL
     RS_RES_ALLOC_PARAMS_INTERNAL *pParams
 )
 {
-    NvHandle                          hClient               = pCallContext->pClient->hClient;
     RsResourceRef                    *pResourceRef          = pCallContext->pResourceRef;
     NvHandle                          hParent               = pResourceRef->pParentRef->hResource;
     NV_VASPACE_ALLOCATION_PARAMETERS *pNvVASpaceAllocParams;
     NvU32                             allocFlags;
     NvU32                             flags                 = 0;
     NvU32                             gpuMask               = 0;
-    NvU32                             gpuMaskInitial        = 0;
     OBJVMM                           *pVmm                  = SYS_GET_VMM(SYS_GET_INSTANCE());
     OBJVASPACE                       *pVAS                  = NULL;
     NvU64                             vasLimit              = 0;
@@ -76,7 +74,7 @@ vaspaceapiConstruct_IMPL
     OBJGPU                           *pGpu                  = GPU_RES_GET_GPU(pVaspaceApi);
     KernelBus                        *pKernelBus            = GPU_GET_KERNEL_BUS(pGpu);
     OBJGPUGRP                        *pGpuGrp               = GPU_RES_GET_GPUGRP(pVaspaceApi);
-    Device                           *pDevice;
+    Device                           *pDevice               = GPU_RES_GET_DEVICE(pVaspaceApi);
     NvBool                            bLockAcquired         = NV_FALSE;
     MemoryManager                    *pMemoryManager        = GPU_GET_MEMORY_MANAGER(pGpu);
     KernelMIGManager                 *pKernelMIGManager     = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
@@ -85,31 +83,15 @@ vaspaceapiConstruct_IMPL
 
     if (RS_IS_COPY_CTOR(pParams))
     {
-        if (!rmGpuGroupLockIsOwner(pGpu->gpuInstance, GPU_LOCK_GRP_ALL, &gpuMask))
-        {
-            //
-            // If we hold some GPU locks already then acquiring more GPU locks
-            // may violate lock ordering and cause dead-lock. To avoid dead-lock in this case,
-            // attempt to take the locks with a conditional acquire.
-            //
-            gpuMaskInitial = rmGpuLocksGetOwnedMask();
-            NvU32 lockFlag = (gpuMaskInitial == 0)
-                ? GPUS_LOCK_FLAGS_NONE
-                : GPU_LOCK_FLAGS_COND_ACQUIRE;
-
-            NV_ASSERT_OK_OR_RETURN(rmGpuGroupLockAcquire(pGpu->gpuInstance,
-                                                         GPU_LOCK_GRP_ALL,
-                                                         lockFlag,
-                                                         RM_LOCK_MODULES_MEM,
-                                                         &gpuMask));
-
-            bLockAcquired = NV_TRUE;
-        }
+        NV_ASSERT_OK_OR_RETURN(rmGpuGroupLockAcquire(pGpu->gpuInstance,
+                                                     GPU_LOCK_GRP_ALL,
+                                                     GPU_LOCK_FLAGS_SAFE_LOCK_UPGRADE,
+                                                     RM_LOCK_MODULES_MEM,
+                                                     &gpuMask));
 
         status = vaspaceapiCopyConstruct_IMPL(pVaspaceApi, pCallContext, pParams);
 
-        if (bLockAcquired)
-            rmGpuGroupLockRelease(gpuMask & (~gpuMaskInitial), GPUS_LOCK_FLAGS_NONE);
+        rmGpuGroupLockRelease(gpuMask, GPUS_LOCK_FLAGS_NONE);
 
         return status;
     }
@@ -124,8 +106,6 @@ vaspaceapiConstruct_IMPL
     // Translate & validate flags
     NV_CHECK_OK_OR_RETURN(LEVEL_WARNING, translateAllocFlagsToVASpaceFlags(allocFlags, &flags));
 
-    NV_ASSERT_OK_OR_RETURN(deviceGetByGpu(pCallContext->pClient, pGpu, NV_TRUE, &pDevice));
-
     //
     // Make sure this GPU is not already locked by this thread
     // Ideally this thread shouldn't have locked any GPU in the system but
@@ -147,7 +127,7 @@ vaspaceapiConstruct_IMPL
         gpuMask = gpumgrGetGpuMask(pGpu);
     }
 
-    status = _vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, hClient, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_RESERVE);
+    status = _vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, pDevice, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_RESERVE);
     NV_ASSERT_OR_RETURN(status == NV_OK, status);
 
     //--------------------------------------------------------------------------------
@@ -195,7 +175,7 @@ vaspaceapiConstruct_IMPL
     {
         MIG_INSTANCE_REF ref;
         NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-            kmigmgrGetInstanceRefFromClient(pGpu, pKernelMIGManager, hClient, &ref),
+            kmigmgrGetInstanceRefFromDevice(pGpu, pKernelMIGManager, pDevice, &ref),
             done);
     }
 
@@ -326,8 +306,7 @@ vaspaceapiConstruct_IMPL
             memmgrIsPmaInitialized(pMemoryManager) &&
             memmgrAreClientPageTablesPmaManaged(pMemoryManager) &&
             !(allocFlags & NV_VASPACE_ALLOCATION_FLAGS_IS_EXTERNALLY_OWNED) &&
-            !(allocFlags & NV_VASPACE_ALLOCATION_FLAGS_IS_FLA) &&
-            !(allocFlags & NV_VASPACE_ALLOCATION_FLAGS_PTETABLE_HEAP_MANAGED))
+            !(allocFlags & NV_VASPACE_ALLOCATION_FLAGS_IS_FLA))
         {
             flags |= VASPACE_FLAGS_PTETABLE_PMA_MANAGED;
         }
@@ -335,25 +314,17 @@ vaspaceapiConstruct_IMPL
         // Get flags for the requested big page size
         flags |= translatePageSizeToVASpaceFlags(pNvVASpaceAllocParams);
 
-        if (0 != originalVaSize)
+        if (originalVaSize != 0)
         {
-            // FLA VASpace can start from any base (!= 0)
-            if (flags & VASPACE_FLAGS_FLA)
+            vasLimit = originalVaBase + originalVaSize - 1;
+            if (vasLimit < originalVaBase)
             {
-                vasLimit = originalVaBase + originalVaSize - 1;
-                if (vasLimit < originalVaBase)
-                {
-                    NV_PRINTF(LEVEL_ERROR,
-                              "Integer overflow !!! Invalid parameters for vaBase:%llx, vaSize:%llx\n",
-                              originalVaBase,
-                              originalVaSize);
-                    status = NV_ERR_INVALID_ARGUMENT;
-                    NV_ASSERT_OR_GOTO(0, done);
-                }
-            }
-            else
-            {
-                vasLimit = originalVaSize - 1;
+                NV_PRINTF(LEVEL_ERROR,
+                          "Integer overflow !!! Invalid parameters for vaBase:%llx, vaSize:%llx\n",
+                          originalVaBase,
+                          originalVaSize);
+                status = NV_ERR_INVALID_ARGUMENT;
+                NV_ASSERT_OR_GOTO(0, done);
             }
         }
 
@@ -394,21 +365,21 @@ vaspaceapiConstruct_IMPL
     NV_PRINTF(LEVEL_INFO,
               "Created vaspaceapi 0x%x, hParent 0x%x, device 0x%x, client 0x%x, varef"
               " 0x%p, parentref 0x%p\n", pResourceRef->hResource, hParent,
-              RES_GET_HANDLE(pDevice), hClient, pResourceRef,
+              RES_GET_HANDLE(pDevice), RES_GET_CLIENT_HANDLE(pDevice), pResourceRef,
               pCallContext->pResourceRef->pParentRef);
 
     // Return the actual VAS base and size.
     pNvVASpaceAllocParams->vaBase = vaspaceGetVaStart(pVAS);
-    pNvVASpaceAllocParams->vaSize = vaspaceGetVaLimit(pVAS) + 1;
+    pNvVASpaceAllocParams->vaSize = (vaspaceGetVaLimit(pVAS) - pNvVASpaceAllocParams->vaBase) + 1;
 
 done:
     if (status == NV_OK)
     {
-        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, hClient, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_TRIM);
+        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, pDevice, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_TRIM);
     }
     else
     {
-        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, hClient, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_RELEASE);
+        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, pDevice, gpuMask, flags, VASPACEAPI_MANAGE_PAGE_LEVELS_RELEASE);
     }
 
     if (bLockAcquired)
@@ -438,14 +409,10 @@ vaspaceapiCopyConstruct_IMPL
     NvHandle                       hParent = pCallContext->pResourceRef->pParentRef->hResource;
     NvHandle                      hVASpace = pCallContext->pResourceRef->hResource;
     OBJVASPACE                       *pVAS = pSrcVaspaceApi->pVASpace;
-    Device                       *pDevice;
     NvBool                          bFlaVA = NV_FALSE;
 
     NV_ASSERT_OR_RETURN(pVAS, NV_ERR_INVALID_ARGUMENT);
 
-    NV_ASSERT_OK_OR_RETURN(deviceGetByGpu(pCallContext->pClient, pGpu, NV_TRUE, &pDevice));
-
-
     bFlaVA = ((IS_VIRTUAL_WITH_SRIOV(pGpu) && !gpuIsWarBug200577889SriovHeavyEnabled(pGpu)) ||
                IS_GSP_CLIENT(pGpu)) &&
               (vaspaceGetFlags(pVAS) & VASPACE_FLAGS_FLA);
@@ -470,9 +437,8 @@ vaspaceapiCopyConstruct_IMPL
               "Shared vaspaceapi 0x%x, device 0x%x, client 0x%x, as vaspace 0x%x for "
               "hParent 0x%x device 0x%x client 0x%x varef 0x%p, deviceref 0x%p\n",
               hVASpaceSrc, pSrcRef->pParentRef->hResource, hClientSrc,
-              hVASpace, hParent, RES_GET_HANDLE(pDevice), hClient,
-              pCallContext->pResourceRef,
-              pCallContext->pResourceRef->pParentRef);
+              hVASpace, hParent, RES_GET_HANDLE(GPU_RES_GET_DEVICE(pVaspaceApi)),
+              hClient, pCallContext->pResourceRef, pCallContext->pResourceRef->pParentRef);
 
     vaspaceIncRefCnt(pVAS);
     pVaspaceApi->pVASpace = pVAS;
@@ -499,8 +465,8 @@ vaspaceapiDestruct_IMPL(VaSpaceApi *pVaspaceApi)
     RsClient           *pRsClient;
     CALL_CONTEXT       *pCallContext;
     RS_RES_FREE_PARAMS_INTERNAL *pParams;
-    Device             *pDevice;
-    NvBool              bBar1VA      =  NV_FALSE;
+    Device             *pDevice      = GPU_RES_GET_DEVICE(pVaspaceApi);
+    NvBool              bBar1VA      = NV_FALSE;
     NvBool              bFlaVA       = NV_FALSE;
 
     GPU_RES_SET_THREAD_BC_STATE(pVaspaceApi);
@@ -511,17 +477,6 @@ vaspaceapiDestruct_IMPL(VaSpaceApi *pVaspaceApi)
     hParent   = pCallContext->pResourceRef->pParentRef->hResource;
     hVASpace  = pCallContext->pResourceRef->hResource;
 
-    status = deviceGetByGpu(pCallContext->pClient, pGpu, NV_TRUE, &pDevice);
-    if (status != NV_OK)
-    {
-        NV_PRINTF(LEVEL_INFO,
-                  "No device found vaspaceapi 0x%x, hParent 0x%x, client 0x%x varef 0x%p,"
-                  " deviceref 0x%p\n", hVASpace, hParent, hClient,
-                  pCallContext->pResourceRef,
-                  pCallContext->pResourceRef->pParentRef);
-        return;
-    }
-
     if (IS_VIRTUAL_WITH_SRIOV(pGpu) && !gpuIsWarBug200577889SriovHeavyEnabled(pGpu))
     {
         bBar1VA = !!(vaspaceGetFlags(pVaspaceApi->pVASpace) & VASPACE_FLAGS_BAR_BAR1);
@@ -536,7 +491,7 @@ vaspaceapiDestruct_IMPL(VaSpaceApi *pVaspaceApi)
     pGVAS = dynamicCast(pVaspaceApi->pVASpace, OBJGVASPACE);
     if (pGVAS != NULL)
     {
-        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, hClient, pVaspaceApi->pVASpace->gpuMask, pGVAS->flags,
+        (void)_vaspaceapiManagePageLevelsForSplitVaSpace(pGpu, pDevice, pVaspaceApi->pVASpace->gpuMask, pGVAS->flags,
                                                          VASPACEAPI_MANAGE_PAGE_LEVELS_RELEASE);
     }
 
@@ -706,7 +661,7 @@ static NvU32 translatePageSizeToVASpaceFlags(NV_VASPACE_ALLOCATION_PARAMETERS *p
  * return directly when that is not the case.
  *
  * @param[in] pGpu      OBJGPU pointer
- * @param[in] hClient   Handle of the client
+ * @param[in] pDevice   Device pointer
  * @param[in] gpuMask   GPU mask
  * @param[in] flags     Flags for the corresponding VA allocation
  * @param[in] action    Requested action to manage the page levels
@@ -715,7 +670,7 @@ static NV_STATUS
 _vaspaceapiManagePageLevelsForSplitVaSpace
 (
     OBJGPU                              *pGpu,
-    NvHandle                             hClient,
+    Device                              *pDevice,
     NvU32                                gpuMask,
     NvU32                                flags,
     VASPACEAPI_MANAGE_PAGE_LEVELS_ACTION action
@@ -753,7 +708,7 @@ _vaspaceapiManagePageLevelsForSplitVaSpace
             // need to thoutoughly test it against SMC sanities before enabling
             // this function on non Client-RM environment.
             //
-            NV_ASSERT_OK_OR_RETURN(memmgrPageLevelPoolsGetInfo(pGpu, pMemoryManager, hClient, &pMemPool));
+            NV_ASSERT_OK_OR_RETURN(memmgrPageLevelPoolsGetInfo(pGpu, pMemoryManager, pDevice, &pMemPool));
 
             if (action == VASPACEAPI_MANAGE_PAGE_LEVELS_RESERVE)
             {
