@@ -838,7 +838,8 @@ spdmDeviceInit_GH100
         return NV_ERR_INSUFFICIENT_RESOURCES;
     }
 
-    status = memdescAlloc(pIOContext->pMemDesc);
+    memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_82, 
+                    pIOContext->pMemDesc);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR, "SPDM: Cannot allocate SPDM surface!\n");
