@@ -337,6 +337,9 @@ memConstructCommon_IMPL
 {
     OBJGPU            *pGpu           = NULL;
     NV_STATUS          status         = NV_OK;
+    NvHandle           hClient        = RES_GET_CLIENT_HANDLE(pMemory);
+    NvHandle           hParent        = RES_GET_PARENT_HANDLE(pMemory);
+    NvHandle           hMemory        = RES_GET_HANDLE(pMemory);
 
     if (pMemDesc == NULL)
         return NV_ERR_INVALID_ARGUMENT;
@@ -472,6 +475,14 @@ memConstructCommon_IMPL
     if (status != NV_OK)
         goto done;
 
+    // Make GSP-RM aware of the memory descriptor so it can be used there
+    if (FLD_TEST_DRF(OS32, _ATTR2, _REGISTER_MEMDESC_TO_PHYS_RM, _TRUE, attr2))
+    {
+        status = memdescRegisterToGSP(pGpu, hClient, hParent, hMemory);
+        if (status != NV_OK)
+            goto done;
+    }
+
     // Initialize the circular list item for tracking dup/sharing of pMemDesc
     pMemory->dupListItem.pNext = pMemory->dupListItem.pPrev = pMemory;
 
@@ -589,6 +600,8 @@ _memDestructCommonWithDevice
                 pFbAllocInfo->hwResId = memdescGetHwResId(pMemory->pMemDesc);
                 pFbAllocInfo->size = pMemory->Length;
                 pFbAllocInfo->format = memdescGetPteKind(pMemory->pMemDesc);
+                pFbAllocInfo->hClient = pRsClient->hClient;
+                pFbAllocInfo->hDevice = hDevice;
 
                 //
                 // Note that while freeing duped memory under a device, the
@@ -765,8 +778,16 @@ memControl_IMPL
 
     if (REF_VAL(NVXXXX_CTRL_CMD_CLASS, pParams->cmd) == NV04_MEMORY)
     {
-        if (pMemory->categoryClassId == NV01_MEMORY_SYSTEM_OS_DESCRIPTOR)
+        //
+        // Tegra SOC import memory usecase uses NV01_MEMORY_SYSTEM_OS_DESCRIPTOR class for
+        // RM resource server registration of memory, RM can return the physical memory attributes
+        // for these imported buffers.
+        //
+        if ((pMemory->categoryClassId == NV01_MEMORY_SYSTEM_OS_DESCRIPTOR) &&
+            (pParams->cmd != NV0041_CTRL_CMD_GET_SURFACE_PHYS_ATTR))
+        {
             return NV_ERR_NOT_SUPPORTED;
+        }
     }
 
     pRmCtrlParams->pGpu = pMemory->pGpu;
@@ -833,7 +854,7 @@ memCopyConstruct_IMPL
 
     if (!!pSrcSubDevice != !!pDstSubDevice)
     {
-        NV_PRINTF(LEVEL_ERROR, "Parent type mismatch between Src and Dst objects"
+        NV_PRINTF(LEVEL_INFO, "Parent type mismatch between Src and Dst objects"
                                "Both should be either device or subDevice\n");
         return NV_ERR_INVALID_OBJECT_PARENT;
     }
