@@ -21,9 +21,6 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
-// FIXME XXX
-#define NVOC_KERNEL_NVLINK_H_PRIVATE_ACCESS_ALLOWED
-
 #include "core/system.h"
 #include "gpu_mgr/gpu_mgr.h"
 #include "kernel/gpu/mig_mgr/kernel_mig_manager.h"
@@ -564,44 +561,21 @@ _p2pCapsGetHostSystemStatusOverPcieBar1
 )
 {
     OBJSYS *pSys = SYS_GET_INSTANCE();
-    NvU32 cpuFamily = DRF_VAL(0000_CTRL, _SYSTEM, _CPU_FAMILY, pSys->cpuInfo.family);
 
-    if (bCommonPciSwitchFound)
+    *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_OK;
+
+    if (bCommonPciSwitchFound ||
+        (pSys->cpuInfo.type == NV0000_CTRL_SYSTEM_CPU_TYPE_RYZEN) ||
+        (pSys->cpuInfo.type == NV0000_CTRL_SYSTEM_CPU_TYPE_XEON_SPR))
     {
-        // Both of GPUs are on the same PCIE switch
-        *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_OK;
         *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_OK;
-
-        NV_PRINTF(LEVEL_INFO, "A common switch detected\n");
-    }
-    else if (cpuFamily == NV0000_CTRL_SYSTEM_CPU_ID_AMD_FAMILY)
-    {
-        *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_OK;
-
-        if (pSys->cpuInfo.type == NV0000_CTRL_SYSTEM_CPU_TYPE_RYZEN)
-            *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_OK;
-        else
-            *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
-
-        NV_PRINTF(LEVEL_INFO, "AMD CPU detected\n");
-    }
-    else if (cpuFamily == NV0000_CTRL_SYSTEM_CPU_ID_INTEL_FAMILY)
-    {
-        *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_OK;
-        *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
-
-        NV_PRINTF(LEVEL_INFO, "Intel CPU detected\n");
     }
     else
     {
-        *pP2PWriteCapStatus = NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
-        *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_NOT_SUPPORTED;
-
-        NV_PRINTF(LEVEL_INFO, "Unsupported HW config\n");
+        *pP2PReadCapStatus = NV0000_P2P_CAPS_STATUS_CHIPSET_NOT_SUPPORTED;
+        NV_PRINTF(LEVEL_INFO, "Unrecognized CPU. Read Cap is disabled\n");
     }
 
-    NV_PRINTF(LEVEL_INFO, "WriteCap 0x%x ReadCap 0x%x\n", *pP2PWriteCapStatus, *pP2PReadCapStatus);
-
     return NV_OK;
 }
 
@@ -637,6 +611,13 @@ _kp2pCapsGetStatusOverPcieBar1
         return NV_ERR_NOT_SUPPORTED;
     }
 
+    // Check if any overrides are enabled.
+    if (_kp2pCapsCheckStatusOverridesForPcie(gpuMask, pP2PWriteCapStatus,
+                                            pP2PReadCapStatus))
+    {
+        return NV_OK;
+    }
+
     //
     // Re-initialize to check loop back configuration if only single GPU in
     // requested mask.
@@ -661,10 +642,10 @@ _kp2pCapsGetStatusOverPcieBar1
     if (*pP2PReadCapStatus == NV0000_P2P_CAPS_STATUS_OK)
         *pP2PReadCapStatus = readCapStatus;
 
-    if ((*pP2PReadCapStatus != NV0000_P2P_CAPS_STATUS_OK) &&
+    if ((*pP2PReadCapStatus != NV0000_P2P_CAPS_STATUS_OK) ||
         (*pP2PWriteCapStatus != NV0000_P2P_CAPS_STATUS_OK))
     {
-        // return not supported if it does not support any operation
+        // return not supported if it does not support both operations
         return NV_ERR_NOT_SUPPORTED;
     }
 
@@ -740,7 +721,7 @@ p2pGetCapsStatus
     while ((pGpu = gpumgrGetNextGpu(gpuMask, &gpuInstance)) != NULL)
     {
         pKernelNvlink = GPU_GET_KERNEL_NVLINK(pGpu);
-        if (pKernelNvlink != NULL && pKernelNvlink->discoveredLinks != 0 &&
+        if (pKernelNvlink != NULL && knvlinkGetDiscoveredLinkMask(pGpu, pKernelNvlink) != 0 &&
             (pSys->getProperty(pSys, PDB_PROP_SYS_NVSWITCH_IS_PRESENT) ||
              knvlinkIsNvswitchProxyPresent(pGpu, pKernelNvlink) ||
              GPU_IS_NVSWITCH_DETECTED(pGpu)))
