@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2004-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2004-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -274,12 +274,7 @@ serverControlApiCopyIn
 
     rmStatus = embeddedParamCopyIn(pEmbeddedParamCopies, pRmCtrlParams);
     if (rmStatus != NV_OK)
-    {
-        rmapiParamsRelease(pParamCopy);
-        pRmCtrlParams->pParams = NvP64_VALUE(pUserParams);
-        pCookie->bFreeParamCopy = NV_FALSE;
         return rmStatus;
-    }
     pCookie->bFreeEmbeddedCopy = NV_TRUE;
 
     return NV_OK;
@@ -404,7 +399,6 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
     RS_LOCK_INFO        lockInfo = {0};
     NvU32 ctrlFlags = 0;
     NvU32 ctrlAccessRight = 0;
-    NvU32 ctrlParamsSize = 0;
     NV_STATUS getCtrlInfoStatus;
 
     RMTRACE_RMAPI(_RMCTRL_ENTRY, cmd);
@@ -492,17 +486,12 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
         }
     }
 
-    getCtrlInfoStatus = rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight, &ctrlParamsSize);
-
     // error check parameters
-    if (((paramsSize != 0) && (pUserParams == (NvP64) 0)) ||
-        ((paramsSize == 0) && (pUserParams != (NvP64) 0))
-        || ((getCtrlInfoStatus == NV_OK) && (paramsSize != ctrlParamsSize))
-        )
+    if (((paramsSize != 0) && (pUserParams == (NvP64) 0))   ||
+        ((paramsSize == 0) && (pUserParams != (NvP64) 0)))
     {
-        NV_PRINTF(LEVEL_WARNING,
-                  "bad params: cmd:0x%x ptr " NvP64_fmt " size: 0x%x expect size: 0x%x\n",
-                  cmd, pUserParams, paramsSize, ctrlParamsSize);
+        NV_PRINTF(LEVEL_WARNING, "bad params: ptr " NvP64_fmt " size: 0x%x\n",
+                  pUserParams, paramsSize);
         rmStatus = NV_ERR_INVALID_ARGUMENT;
         goto done;
     }
@@ -529,6 +518,7 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
         lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
     }
 
+    getCtrlInfoStatus = rmapiutilGetControlInfo(cmd, &ctrlFlags, &ctrlAccessRight);
     if (getCtrlInfoStatus == NV_OK)
     {
         //
@@ -606,20 +596,9 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
             {
                 rmCtrlParams.pCookie->apiCopyFlags |= RMCTRL_API_COPY_FLAGS_FORCE_SKIP_COPYOUT_ON_ERROR;
 
-                rmStatus = serverControlApiCopyIn(&g_resServ, &rmCtrlParams,
-                                                  rmCtrlParams.pCookie);
-                if (rmStatus == NV_OK)
-                {
-                    rmStatus = rmapiControlCacheGet(hClient, hObject, cmd,
-                                                    rmCtrlParams.pParams,
-                                                    paramsSize);
-
-                    // rmStatus is passed in for error handling
-                    rmStatus = serverControlApiCopyOut(&g_resServ,
-                                                       &rmCtrlParams,
-                                                       rmCtrlParams.pCookie,
-                                                       rmStatus);
-                }
+                serverControlApiCopyIn(&g_resServ, &rmCtrlParams, rmCtrlParams.pCookie);
+                rmStatus = rmapiControlCacheGet(hClient, hObject, cmd, rmCtrlParams.pParams, paramsSize);
+                serverControlApiCopyOut(&g_resServ, &rmCtrlParams, rmCtrlParams.pCookie, rmStatus);
 
                 if (rmStatus == NV_OK)
                 {
@@ -641,10 +620,29 @@ _rmapiRmControl(NvHandle hClient, NvHandle hObject, NvU32 cmd, NvP64 pUserParams
         RM_API_CONTEXT rmApiContext = {0};
         rmStatus = rmapiPrologue(pRmApi, &rmApiContext);
         if (rmStatus != NV_OK)
-            goto done;
+            goto epilogue;
+
+        //
+        // If this is an internal request within the same RM instance, make
+        // sure we don't double lock clients and preserve previous lock state.
+        //
+        if (bInternalRequest && resservGetTlsCallContext() != NULL)
+        {
+            rmStatus = rmapiInitLockInfo(pRmApi, hClient, NV01_NULL_OBJECT, &lockInfo);
+            if (rmStatus != NV_OK)
+                goto epilogue;
+
+            //
+            // rmapiInitLockInfo overwrites lockInfo.flags, re-add
+            // RM_LOCK_FLAGS_NO_API_LOCK if it was originally added.
+            //
+            if (pRmApi->bApiLockInternal)
+                lockInfo.flags |= RM_LOCK_FLAGS_NO_API_LOCK;
+        }
 
         lockInfo.flags |= RM_LOCK_FLAGS_RM_SEMA;
         rmStatus = serverControl(&g_resServ, &rmCtrlParams);
+epilogue:
         rmapiEpilogue(pRmApi, &rmApiContext);
     }
 done:
@@ -858,14 +856,12 @@ serverControlLookupLockFlags
 {
     //
     // Calls with LOCK_TOP doesn't fill in the cookie param correctly.
-    // This is just a WAR for this. Since the optimisation that depends on this
-    // flag only works in full GSP offload mode, it's gated to that.
+    // This is just a WAR for this.
     //
-    NvBool areAllGpusInOffloadMode = gpumgrAreAllGpusInOffloadMode();
     NvU32 controlFlags = pRmCtrlExecuteCookie->ctrlFlags;
-    if (controlFlags == 0 && !RMCFG_FEATURE_PLATFORM_GSP && areAllGpusInOffloadMode)
+    if (controlFlags == 0 && !RMCFG_FEATURE_PLATFORM_GSP)
     {
-        NV_STATUS status = rmapiutilGetControlInfo(pRmCtrlParams->cmd, &controlFlags, NULL, NULL);
+        NV_STATUS status = rmapiutilGetControlInfo(pRmCtrlParams->cmd, &controlFlags, NULL);
         if (status != NV_OK)
         {
             NV_PRINTF(LEVEL_INFO,
@@ -874,6 +870,8 @@ serverControlLookupLockFlags
         }
     }
 
+    NvBool areAllGpusInOffloadMode = gpumgrAreAllGpusInOffloadMode();
+
     //
     // If the control is ROUTE_TO_PHYSICAL, and we're in GSP offload mode,
     // we can use a more relaxed locking mode:
